@@ -1,14 +1,14 @@
 /*
- * parallel.h
+ * atomic.h
  *
  *  Created on: 2014/07/04
  *      Author: utsugi
  */
 
-#ifndef PRIVATE2_ATOMIC_H_
-#define PRIVATE2_ATOMIC_H_
+#ifndef PRIVATE_ATOMIC_H
+#define PRIVATE_ATOMIC_H
 
 void	atomic_add (double *data, double delta);
 void	atomic_max (double *data, double val);
 
-#endif /* PRIVATE2_ATOMIC_H_ */
+#endif /* PRIVATE_ATOMIC_H */
