@@ -24,7 +24,6 @@ bic_info_alloc (void)
 	info->n = 0.;
 	info->rss = 0.;
 	info->df = 0.;
-	info->gamma = 0.;
 	info->bic_val = CDESCENT_POSINF;
 	return info;
 }
@@ -38,9 +37,11 @@ calc_rss (const cdescent *cd)
 	mm_dense	*r = mm_real_copy (cd->lreg->y);	// r = y
 	mm_real_axjpy (-1., cd->mu, 0, r);	// r = y - mu
 	// intercept
-	if (fabs (cd->b) > 0.) mm_real_xj_add_const (r, 0, - cd->b);	// r = y - mu - b
+	if (cd->use_intercept && fabs (cd->b0) > 0.) mm_real_xj_add_const (r, 0, - cd->b0);	// r = y - mu - b
 	rss = mm_real_xj_ssq (r, 0);	// rss = | y - mu |^2
 	mm_real_free (r);
+	// if not lasso, rss = | y - mu |^2 + lambda2 * | nu |^2
+	// if (!cd->is_regtype_lasso && cd->nu) rss += cd->lambda2 * mm_real_xj_ssq (cd->nu, 0);
 	return rss;
 }
 
@@ -60,7 +61,7 @@ calc_degree_of_freedom (const cdescent *cd)
 		if (fabs (cd->beta->data[j]) > 0.) {
 			if (cd->is_regtype_lasso) df += 1.;
 			else {
-				double	gj = cd->lreg->lambda2 * cd->lreg->dtd[j];
+				double	gj = cd->lambda2 * cd->lreg->dtd[j];
 				if (!cd->lreg->xnormalized) gj /= cd->lreg->xtx[j];
 				df += 1. / (1. + gj);
 			}
@@ -69,30 +70,50 @@ calc_degree_of_freedom (const cdescent *cd)
 	return df;
 }
 
-/*** Extended Bayesian Information Criterion (Chen and Chen, 2008)
- * eBIC = log(rss) + df * ( log(m) + 2 * gamma * log(n) ) / m
- * gamma	: tuning parameter for eBIC
+/*** default function which evaluates BIC as: BIC = log (rss) + log (m) * df / m ***/
+double
+cdescent_default_bic_eval_func (const cdescent *cd, bic_info *info, void *data)
+{
+	double	val = log (info->rss) + info->df * log (info->m) / info->m;
+	return val;
+}
+
+static bic_func *
+bic_function_alloc (void)
+{
+	bic_func	*func = (bic_func *) malloc (sizeof (bic_func));
+	func->function = NULL;
+	func->data = NULL;
+	return func;
+}
+
+/*** create bic_function object ***/
+bic_func *
+bic_function_new (const bic_eval_func function, void *data)
+{
+	bic_func	*func = bic_function_alloc ();
+	func->function = function;
+	func->data = data;
+	return func;
+}
+
+/*** Bayesian Information Criterion
+ * BIC = log(rss) + df * log(m) / m
  * rss		: residual sum of squares |b - Z * beta|^2
  * df		: degree of freedom
  * m		: number of data (number of rows of b and Z)
- * n		: number of variables (number of columns of Z and number of rows of beta)
- * if gamma = 0, eBIC is identical with the classical BIC ***/
+ * n		: number of variables (number of columns of Z and number of rows of beta) ***/
 bic_info *
-cdescent_eval_bic (const cdescent *cd, const double gamma)
+calc_bic_info (const cdescent *cd)
 {
+	bic_func	*bicfunc = cd->path->bicfunc;
 	bic_info	*info;
-	if (gamma < 0.) {
-		printf_warning ("cdescent_eval_bic", "gamma must be >= 0.", __FILE__, __LINE__);
-		return NULL;
-	}
 	info = bic_info_alloc ();
-	info->gamma = gamma;
 	info->rss = calc_rss (cd);
 	info->df = calc_degree_of_freedom (cd);
-	info->m = (double) cd->lreg->x->m;
-	info->n = (double) cd->lreg->x->n;
-	info->bic_val = log (info->rss) + info->df * log (info->m) / info->m;
-	if (fabs (gamma) > 0.) info->bic_val += 2. * info->df * info->gamma * log (info->n) / info->m;
+	info->m = (double) *cd->m;
+	info->n = (double) *cd->n;
+	info->bic_val = bicfunc->function (cd, info, bicfunc->data);
 	return info;
 }
 
