@@ -195,14 +195,66 @@ mm_real_set_lower (mm_real *x)
 	return;
 }
 
-/* copy sparse */
-static mm_sparse *
-mm_real_copy_sparse (const mm_sparse *src)
+/*** sort each columns of mm_sparse ***/
+typedef struct {
+	int		i;
+	double	data;
+} matrix_element;
+
+int
+compare_row_index (const void *a, const void *b)
+{
+	matrix_element	*_a = (matrix_element *) a;
+	matrix_element	*_b = (matrix_element *) b;
+	return _a->i - _b->i;
+}
+
+static void
+mm_real_sort_sparse (mm_sparse *s)
+{
+	int		j;
+	int		*si = s->i;
+	double	*sdata = s->data;
+	int		*sp = s->p;
+	matrix_element	t[s->m];
+
+	for (j = 0; j < s->n; j++) {
+		int		k;
+		int		p = sp[j];
+		int		n = sp[j + 1] - p;
+
+		if (n < 2) continue;
+
+		for (k = 0; k < n; k++) {
+			t[k].i = si[k + p];
+			t[k].data = sdata[k + p];
+		}
+
+		qsort (t, n, sizeof (matrix_element), compare_row_index);
+
+		for (k = 0; k < n; k++) {
+			si[k + p] = t[k].i;
+			sdata[k + p] = t[k].data;
+		}
+
+	}
+	return;
+}
+
+void
+mm_real_sort (mm_real *x)
+{
+	if (mm_real_is_sparse (x)) mm_real_sort_sparse (x);
+	return;
+}
+
+/* memcpy sparse */
+static void
+mm_real_memcpy_sparse (mm_sparse *dest, const mm_sparse *src)
 {
 	int			k;
 	int			n = src->n;
 	int			nnz = src->nnz;
-	mm_sparse	*dest = mm_real_new (MM_REAL_SPARSE, src->symm, src->m, src->n, src->nnz);
 
 	int			*si = src->i;
 	int			*sp = src->p;
@@ -213,6 +265,38 @@ mm_real_copy_sparse (const mm_sparse *src)
 	for (k = 0; k <= n; k++) dp[k] = sp[k];
 	dcopy_ (&nnz, src->data, &ione, dest->data, &ione);
 
+	return;
+}
+
+/* memcpy dense */
+static mm_dense *
+mm_real_memcpy_dense (mm_dense *dest, const mm_dense *src)
+{
+	dcopy_ (&src->nnz, src->data, &ione, dest->data, &ione);
+	return dest;
+}
+
+/*** memcpy mm_real ***/
+void
+mm_real_memcpy (mm_real *dest, const mm_real *src)
+{
+	if (mm_real_is_sparse (src)) {
+		if (!mm_real_is_sparse (dest)) error_and_exit ("mm_real_memcpy", "destination matrix format does not match source matrix format.", __FILE__, __LINE__);
+		mm_real_memcpy_sparse (dest, src);
+	} else {
+		if (!mm_real_is_dense (dest)) error_and_exit ("mm_real_memcpy", "destination matrix format does not match source matrix format.", __FILE__, __LINE__);
+		mm_real_memcpy_dense (dest, src);
+	}
+	return;
+}
+
+
+/* copy sparse */
+static mm_sparse *
+mm_real_copy_sparse (const mm_sparse *src)
+{
+	mm_sparse	*dest = mm_real_new (MM_REAL_SPARSE, src->symm, src->m, src->n, src->nnz);
+	mm_real_memcpy_sparse (dest, src);
 	return dest;
 }
 
@@ -221,7 +305,7 @@ static mm_dense *
 mm_real_copy_dense (const mm_dense *src)
 {
 	mm_dense	*dest = mm_real_new (MM_REAL_DENSE, src->symm, src->m, src->n, src->nnz);
-	dcopy_ (&src->nnz, src->data, &ione, dest->data, &ione);
+	mm_real_memcpy_dense (dest, src);
 	return dest;
 }
 
@@ -232,26 +316,9 @@ mm_real_copy (const mm_real *x)
 	return (mm_real_is_sparse (x)) ? mm_real_copy_sparse (x) : mm_real_copy_dense (x);
 }
 
-/* set all elements of array to val */
-static void
-mm_real_array_set_all (const int n, double *data, const double val)
-{
-	int		k;
-	for (k = 0; k < n; k++) data[k] = val;
-	return;
-}
-
-/*** set x->data to val ***/
-void
-mm_real_set_all (mm_real *x, const double val)
-{
-	mm_real_array_set_all (x->nnz, x->data, val);
-	return;
-}
-
 /*** convert sparse -> dense ***/
 mm_dense *
-mm_real_sparse_to_dense (const mm_sparse *s)
+mm_real_copy_sparse_to_dense (const mm_sparse *s)
 {
 	int			j, k;
 	int			p, n;
@@ -281,7 +348,7 @@ mm_real_sparse_to_dense (const mm_sparse *s)
 /*** convert dense -> sparse
  * if fabs (x->data[j]) < threshold, set to 0 ***/
 mm_sparse *
-mm_real_dense_to_sparse (const mm_dense *d, const double threshold)
+mm_real_copy_dense_to_sparse (const mm_dense *d, const double threshold)
 {
 	int			i, j, k;
 	mm_sparse	*s;
@@ -346,28 +413,164 @@ mm_real_dense_to_sparse (const mm_dense *d, const double threshold)
 	return s;
 }
 
+/* set all elements of array to val */
+static void
+mm_real_array_set_all (const int n, double *data, const double val)
+{
+	int		k;
+	for (k = 0; k < n; k++) data[k] = val;
+	return;
+}
+
+/*** set x->data to val ***/
+void
+mm_real_set_all (mm_real *x, const double val)
+{
+	mm_real_array_set_all (x->nnz, x->data, val);
+	return;
+}
+
+/* convert sparse to dense */
+bool
+mm_real_sparse_to_dense (mm_sparse *s)
+{
+	int		j, k;
+
+	int		m;
+	int		n;
+	int		nnz;
+
+	double	*tmp_data;
+
+	double	*td;
+	int		*si;
+	int		*sp;
+	double	*sd;
+
+	if (!mm_real_is_sparse (s)) return false;
+
+	m = s->m;
+	n = s->n;
+	nnz = m * n;
+
+	/* copy s->data */
+	tmp_data = (double *) malloc (s->nnz * sizeof (double));
+	for (k = 0; k < s->nnz; k++) tmp_data[k] = s->data[k];
+	td = tmp_data;
+
+	/* reallocate s->data */
+	free (s->data);
+	s->data = (double *) malloc (nnz * sizeof (double));
+	mm_real_array_set_all (nnz, s->data, 0.);
+
+	/* convert sparse to dense */
+	si = s->i;
+	sp = s->p;
+	sd = s->data;
+	for (j = 0; j < n; j++) {
+		int		np = sp[1] - *sp;
+		for (k = 0; k < np; k++) {
+			sd[*si] = *td;
+			si++;
+			td++;
+		}
+		sd += s->m;
+		sp++;
+	}
+	free (tmp_data);
+
+	mm_set_array (&s->typecode);
+	s->nnz = nnz;
+	free (s->i);
+	s->i = NULL;
+	free (s->p);
+	s->p = NULL;
+
+	return true;
+}
+
+/* convert dense to sparse */
+bool
+mm_real_dense_to_sparse (mm_dense *d, const double threshold)
+{
+	int		i, j, k;
+
+	int		m;
+	int		n;
+	int		nnz;
+
+	int		*di;
+	int		*dp;
+	double	*dd;
+	double	*d0;
+
+	if (!mm_real_is_dense (d)) return false;
+
+	m = d->m;
+	n = d->n;
+	nnz = d->nnz;
+
+	/* convert dense to sparse */
+	d->i = (int *) malloc (nnz * sizeof (int));
+	d->p = (int *) malloc ((n + 1) * sizeof (int));
+	di = d->i;
+	dp = d->p;
+	dd = d->data;
+	d0 = d->data;
+
+	k = 0;
+	*dp = 0;
+	for (j = 0; j < n; j++) {
+		for (i = 0; i < m; i++) {
+			double	val = *d0;
+			if (fabs (val) > threshold) {
+				*(di++) = i;
+				*(dd++) = val;
+				k++;
+			}
+			d0++;
+		}
+		*(++dp) = k;
+	}
+
+	mm_set_coordinate (&d->typecode);
+	if (k != d->nnz) mm_real_realloc (d, k);
+
+	return true;
+}
+
+/* binary search: search the value key from array *s of length n.
+ * if found, return its index otherwise -1 */
+static int
+bin_search (const int key, const int *s, const int n)
+{
+	int	start = 0;
+	int	end = n - 1;
+	int	mid;
+	while (start < end) {
+		mid = (start + end) / 2;
+		if (key <= s[mid]) end = mid;
+		else start = mid + 1;
+	}
+	return (key == s[end]) ? end : -1;
+}
+
 /* find element that s->i[l] = j in the k-th column of s and return its index l */
 static int
-find_jth_row_element_of_sk (const int j, const mm_sparse *s, const int k)
+find_row_element (const int j, const mm_sparse *s, const int k)
 {
-	int		l;
 	int		p = s->p[k];
 	int		n = s->p[k + 1] - p;
 	int		*si = s->i + p;
-	for (l = 0; l < n; l++) {
-		int		sil = si[l];
-		if (sil < j) continue;
-		if (sil == j) return l + p;	// found
-		else break;
-	}
-	return -1;	// not found
+	int		res = bin_search (j, si, n);
+	return (res < 0) ? -1 : res + p;
 }
 
 /* convert sparse symmetric -> sparse general */
 static mm_sparse *
 mm_real_symmetric_to_general_sparse (const mm_sparse *x)
 {
-	int			j, m;
+	int			i, j;
 	mm_sparse	*s;
 
 	int			*si;
@@ -385,69 +588,77 @@ mm_real_symmetric_to_general_sparse (const mm_sparse *x)
 	sp = s->p;
 	sd = s->data;
 
-	m = 0;
+	i = 0;
 	for (j = 0; j < xn; j++) {
 		int		k;
 		int		pend = xp[j + 1];
 		if (mm_real_is_upper (x)) {
 			for (k = xp[j]; k < pend; k++) {
-				si[m] = xi[k];
-				sd[m++] = xd[k];
+				si[i] = xi[k];
+				sd[i++] = xd[k];
 			}
 			for (k = j + 1; k < xn; k++) {
-				int		l = find_jth_row_element_of_sk (j, x, k);
+				int		l = find_row_element (j, x, k);
 				// if found
 				if (l >= 0) {
-					si[m] = k;
-					sd[m++] = xd[l];
+					si[i] = k;
+					sd[i++] = xd[l];
 				}
 			}
 		} else if (mm_real_is_lower (x)) {
 			for (k = 0; k < j; k++) {
-				int		l = find_jth_row_element_of_sk (j, x, k);
+				int		l = find_row_element (j, x, k);
 				// if found
 				if (l >= 0) {
-					si[m] = k;
-					sd[m++] = xd[l];
+					si[i] = k;
+					sd[i++] = xd[l];
 				}
 			}
 			for (k = xp[j]; k < pend; k++) {
-				si[m] = xi[k];
-				sd[m++] = xd[k];
+				si[i] = xi[k];
+				sd[i++] = xd[k];
 			}
 		}
-		sp[j + 1] = m;
+		sp[j + 1] = i;
 	}
-	if (s->nnz != m) mm_real_realloc (s, m);
+	if (s->nnz != i) mm_real_realloc (s, i);
 	return s;
 }
 
 /* convert dense symmetric -> dense general */
-static mm_dense *
-mm_real_symmetric_to_general_dense (const mm_dense *x)
+static void
+mm_real_symmetric_to_general_dense (mm_dense *d)
 {
 	int			j;
-	mm_dense	*d = mm_real_copy (x);
-	if (!mm_real_is_symmetric (x)) return d;
-
-	if (mm_real_is_upper (x)) {
-		for (j = 0; j < x->n; j++) {
+	if (mm_real_is_upper (d)) {
+		for (j = 0; j < d->n; j++) {
 			int		i0 = j + 1;
-			int		n = x->m - i0;
-			dcopy_ (&n, x->data + j + i0 * x->m, &x->m, d->data + i0 + j * d->m, &ione);
+			int		n = d->m - i0;
+			dcopy_ (&n, d->data + j + i0 * d->m, &d->m, d->data + i0 + j * d->m, &ione);
 		}
 	} else {
-		for (j = 0; j < x->n; j++) dcopy_ (&j, x->data + j, &x->m, d->data + j * d->m, &ione);
+		for (j = 0; j < d->n; j++) dcopy_ (&j, d->data + j, &d->m, d->data + j * d->m, &ione);
 	}
 	mm_real_set_general (d);
-	return d;
+	return;
 }
 
 /* convert symmetric -> general */
-mm_real *
-mm_real_symmetric_to_general (const mm_real *x)
+bool
+mm_real_symmetric_to_general (mm_real *x)
 {
-	return (mm_real_is_sparse (x)) ? mm_real_symmetric_to_general_sparse (x) : mm_real_symmetric_to_general_dense (x);
+	if (!mm_real_is_symmetric (x)) return false;
+
+	if (mm_real_is_sparse (x)) {
+		mm_sparse	*s = mm_real_symmetric_to_general_sparse (x);
+		if (s->nnz != x->nnz) mm_real_realloc (x, s->nnz);
+		mm_real_memcpy_sparse (x, s);
+		mm_real_free (s);
+	} else {
+		mm_real_symmetric_to_general_dense (x);
+	}
+
+	return true;
 }
 
 /* identity sparse matrix */
@@ -668,7 +879,7 @@ mm_real_sj_asum (const mm_sparse *s, const int j)
 			k1 = j;
 		}
 		for (k = k0; k < k1; k++) {
-			int		l = find_jth_row_element_of_sk (j, s, k);
+			int		l = find_row_element (j, s, k);
 			// if found
 			if (l >= 0) asum += fabs (sd[l]);
 		}
@@ -729,7 +940,7 @@ mm_real_sj_sum (const mm_sparse *s, const int j)
 		}
 		sd = s->data;
 		for (k = k0; k < k1; k++) {
-			int		l = find_jth_row_element_of_sk (j, s, k);
+			int		l = find_row_element (j, s, k);
 			// if found
 			if (l >= 0) sum += sd[l];
 		}
@@ -796,7 +1007,7 @@ mm_real_sj_ssq (const mm_sparse *s, const int j)
 			k1 = j;
 		}
 		for (k = k0; k < k1; k++) {
-			int		l = find_jth_row_element_of_sk (j, s, k);
+			int		l = find_row_element (j, s, k);
 			// if found
 			if (l >= 0) ssq += pow (sd[l], 2.);
 		}
@@ -846,21 +1057,22 @@ mm_real_xj_nrm2 (const mm_real *x, const int j)
 	return sqrt (ssq);
 }
 
-/* z = alpha * s * y + beta * z, where s is sparse matrix and y is dense vector */
+/* z = alpha * s * y(:,k) + beta * z, where s is sparse matrix and y is dense general */
 static void
-mm_real_s_dot_y (bool trans, const double alpha, const mm_sparse *s, const mm_dense *y, const double beta, mm_dense *z)
+mm_real_s_dot_yk (const bool trans, const double alpha, const mm_sparse *s, const mm_dense *y, const int k, const double beta, mm_dense *z)
 {
-	int			j, k;
-	int			p, n;
+	int			j, l;
+	int			p, m, n;
 
 	int			*si;
 	double		*sd;
 	int			*sp = s->p;
-	double		*yd = y->data;
-	double		*zd = z->data;
+	double		*yk = y->data + k * y->m;
+	double		*zk = z->data + k * z->m;
 
-	if (fabs (beta) > 0.) mm_real_xj_scale (z, 0, beta);
-	else mm_real_set_all (z, 0.);
+	m = (!trans) ? s->m : s->n;
+	if (fabs (beta) > DBL_EPSILON) dscal_ (&m, &beta, zk, &ione);
+	else mm_real_array_set_all (m, zk, 0.);
 
 	if (trans) {
 		if (!mm_real_is_symmetric (s)) {
@@ -870,48 +1082,48 @@ mm_real_s_dot_y (bool trans, const double alpha, const mm_sparse *s, const mm_de
 				if (n <= 0) continue;
 				si = s->i + p;
 				sd = s->data + p;
-				for (k = 0; k < n; k++) zd[j] += alpha * sd[k] * yd[si[k]];
+				for (l = 0; l < n; l++) zk[j] += alpha * sd[l] * yk[si[l]];
 			}
 		} else if (mm_real_is_upper (s)) {
-			int		sik;
+			int		sil;
 			double	alpha_sdk;
 			for (j = 0; j < s->n; j++) {
 				p = sp[j];
 				n = sp[j + 1] - p;
 				si = s->i + p;
 				sd = s->data + p;
-				for (k = 0; k < n - 1; k++) {
-					sik = si[k];
-					alpha_sdk = alpha * sd[k];
-					zd[j] += alpha_sdk * yd[sik];
-					zd[sik] += alpha_sdk * yd[j];
+				for (l = 0; l < n - 1; l++) {
+					sil = si[l];
+					alpha_sdk = alpha * sd[l];
+					zk[j] += alpha_sdk * yk[sil];
+					zk[sil] += alpha_sdk * yk[j];
 				}
 				if (n <= 0) continue;
-				sik = si[n - 1];
+				sil = si[n - 1];
 				alpha_sdk = alpha * sd[n - 1];
-				zd[j] += alpha_sdk * yd[sik];
+				zk[j] += alpha_sdk * yk[sil];
 				// omit if diagonal element
-				if (j != sik) zd[sik] += alpha_sdk * yd[j];
+				if (j != sil) zk[sil] += alpha_sdk * yk[j];
 			}
 		} else {
-			int		sik;
-			double	alpha_sdk;
+			int		sil;
+			double	alpha_sdl;
 			for (j = 0; j < s->n; j++) {
 				p = sp[j];
 				n = sp[j + 1] - p;
 				if (n <= 0) continue;
 				si = s->i + p;
 				sd = s->data + p;
-				sik = si[0];
-				alpha_sdk = alpha * sd[0];
-				zd[j] += alpha_sdk * yd[sik];
+				sil = si[0];
+				alpha_sdl = alpha * sd[0];
+				zk[j] += alpha_sdl * yk[sil];
 				// omit if diagonal element
-				if (j != sik) zd[sik] += alpha_sdk * yd[j];
-				for (k = 1; k < n; k++) {
-					sik = si[k];
-					alpha_sdk = alpha * sd[k];
-					zd[j] += alpha_sdk * yd[sik];
-					zd[sik] += alpha_sdk * yd[j];
+				if (j != sil) zk[sil] += alpha_sdl * yk[j];
+				for (l = 1; l < n; l++) {
+					sil = si[l];
+					alpha_sdl = alpha * sd[l];
+					zk[j] += alpha_sdl * yk[sil];
+					zk[sil] += alpha_sdl * yk[j];
 				}
 			}
 		}
@@ -923,48 +1135,48 @@ mm_real_s_dot_y (bool trans, const double alpha, const mm_sparse *s, const mm_de
 				if (n <= 0) continue;
 				si = s->i + p;
 				sd = s->data + p;
-				for (k = 0; k < n; k++) zd[si[k]] += alpha * sd[k] * yd[j];
+				for (l = 0; l < n; l++) zk[si[l]] += alpha * sd[l] * yk[j];
 			}
 		} else if (mm_real_is_upper (s)) {
-			int		sik;
-			double	alpha_sdk;
+			int		sil;
+			double	alpha_sdl;
 			for (j = 0; j < s->n; j++) {
 				p = sp[j];
 				n = sp[j + 1] - p;
 				si = s->i + p;
 				sd = s->data + p;
-				for (k = 0; k < n - 1; k++) {
-					sik = si[k];
-					alpha_sdk = alpha * sd[k];
-					zd[sik] += alpha_sdk * yd[j];
-					zd[j] += alpha_sdk * yd[sik];
+				for (l = 0; l < n - 1; l++) {
+					sil = si[l];
+					alpha_sdl = alpha * sd[l];
+					zk[sil] += alpha_sdl * yk[j];
+					zk[j] += alpha_sdl * yk[sil];
 				}
 				if (n <= 0) continue;
-				sik = si[n - 1];
-				alpha_sdk = alpha * sd[n - 1];
-				zd[sik] += alpha_sdk * yd[j];
+				sil = si[n - 1];
+				alpha_sdl = alpha * sd[n - 1];
+				zk[sil] += alpha_sdl * yk[j];
 				// omit if diagonal element
-				if (j != sik) zd[j] += alpha_sdk * yd[sik];
+				if (j != sil) zk[j] += alpha_sdl * yk[sil];
 			}
 		} else {
-			int		sik;
-			double	alpha_sdk;
+			int		sil;
+			double	alpha_sdl;
 			for (j = 0; j < s->n; j++) {
 				p = sp[j];
 				n = sp[j + 1] - p;
 				if (n <= 0) continue;
 				si = s->i + p;
 				sd = s->data + p;
-				sik = si[0];
-				alpha_sdk = alpha * sd[0];
-				zd[sik] += alpha_sdk * yd[j];
+				sil = si[0];
+				alpha_sdl = alpha * sd[0];
+				zk[sil] += alpha_sdl * yk[j];
 				// omit if diagonal element
-				if (j != sik) zd[j] += alpha_sdk * yd[sik];
-				for (k = 1; k < n; k++) {
-					sik = si[k];
-					alpha_sdk = alpha * sd[k];
-					zd[sik] += alpha_sdk * yd[j];
-					zd[j] += alpha_sdk * yd[sik];
+				if (j != sil) zk[j] += alpha_sdl * yk[sil];
+				for (l = 1; l < n; l++) {
+					sil = si[l];
+					alpha_sdl = alpha * sd[l];
+					zk[sil] += alpha_sdl * yk[j];
+					zk[j] += alpha_sdl * yk[sil];
 				}
 			}
 		}
@@ -972,131 +1184,138 @@ mm_real_s_dot_y (bool trans, const double alpha, const mm_sparse *s, const mm_de
 	return;
 }
 
-/* z = alpha * d * y + beta * z, where d is dense matrix and y is dense vector */
+/* z = alpha * d * y(:,k) + beta * z, where d is dense matrix and y is dense general */
 static void
-mm_real_d_dot_y (bool trans, const double alpha, const mm_dense *d, const mm_dense *y, const double beta, mm_dense *z)
+mm_real_d_dot_yk (const bool trans, const double alpha, const mm_dense *d, const mm_dense *y, const int k, const double beta, mm_dense *z)
 {
+	double	*yk = y->data + k * y->m;
+	double	*zk = z->data + k * z->m;
 	if (!mm_real_is_symmetric (d)) {
 		// z = alpha * d * y + beta * z
-		dgemv_ ((trans) ? "T" : "N", &d->m, &d->n, &alpha, d->data, &d->m, y->data, &ione, &beta, z->data, &ione);
+		dgemv_ ((trans) ? "T" : "N", &d->m, &d->n, &alpha, d->data, &d->m, yk, &ione, &beta, zk, &ione);
 	} else {
 		char	uplo = (mm_real_is_upper (d)) ? 'U' : 'L';
 		// z = alpha * d * y + beta * z
-		dsymv_ (&uplo, &d->m, &alpha, d->data, &d->m, y->data, &ione, &beta, z->data, &ione);
+		dsymv_ (&uplo, &d->m, &alpha, d->data, &d->m, yk, &ione, &beta, zk, &ione);
 	}
 	return;
 }
 
-/*** alpha * x * y, where x is sparse/dense matrix and y is dense vector ***/
-void
-mm_real_x_dot_y (bool trans, const double alpha, const mm_real *x, const mm_dense *y, const double beta, mm_dense *z)
+/*** alpha * x * y(:,k), where x is sparse/dense matrix and y is dense general ***/
+static void
+mm_real_x_dot_yk (const bool trans, const double alpha, const mm_real *x, const mm_dense *y, const int k, const double beta, mm_dense *z)
 {
+	return (mm_real_is_sparse (x)) ? mm_real_s_dot_yk (trans, alpha, x, y, k, beta, z) : mm_real_d_dot_yk (trans, alpha, x, y, k, beta, z);
+}
+
+/*** alpha * x * y, where x is sparse/dense matrix and y is dense matrix ***/
+void
+mm_real_x_dot_y (const bool trans, const double alpha, const mm_real *x, const mm_dense *y, const double beta, mm_dense *z)
+{
+	int		k;
 	if (!mm_real_is_dense (y)) error_and_exit ("mm_real_x_dot_y", "y must be dense.", __FILE__, __LINE__);
 	if (mm_real_is_symmetric (y)) error_and_exit ("mm_real_x_dot_y", "y must be general.", __FILE__, __LINE__);
-	if (y->n != 1) error_and_exit ("mm_real_x_dot_y", "y must be vector.", __FILE__, __LINE__);
 	if ((trans && x->m != y->m) || (!trans && x->n != y->m))
 		error_and_exit ("mm_real_x_dot_y", "dimensions of x and y do not match.", __FILE__, __LINE__);
 	if (!mm_real_is_dense (z)) error_and_exit ("mm_real_x_dot_y", "z must be dense.", __FILE__, __LINE__);
 	if (mm_real_is_symmetric (z)) error_and_exit ("mm_real_x_dot_y", "z must be general.", __FILE__, __LINE__);
-	if (z->n != 1) error_and_exit ("mm_real_x_dot_y", "z must be vector.", __FILE__, __LINE__);
 	if ((trans && x->n != z->m) || (!trans && x->m != z->m))
 		error_and_exit ("mm_real_x_dot_y", "dimensions of x and z do not match.", __FILE__, __LINE__);
-
-	return (mm_real_is_sparse (x)) ? mm_real_s_dot_y (trans, alpha, x, y, beta, z) : mm_real_d_dot_y (trans, alpha, x, y, beta, z);
+#pragma omp parallel for
+	for (k = 0; k < y->n; k++) mm_real_x_dot_yk (trans, alpha, x, y, k, beta, z);
+	return;
 }
 
-/* s(:,j)' * y */
+/* s(:,j)' * y(:,k) */
 static double
-mm_real_sj_trans_dot_y (const mm_sparse *s, const int j, const mm_dense *y)
+mm_real_sj_trans_dot_yk (const mm_sparse *s, const int j, const mm_dense *y, const int k)
 {
 	double	val = 0;
 
 	int		*si;
 	double	*sd;
-	double	*yd = y->data;
+	double	*yk = y->data + k * y->m;
 
-	int		k;
+	int		l;
 	int		p = s->p[j];
 	int		n = s->p[j + 1] - p;
-	int		mod = n % 4;
 
-	// unrolling
 	si = s->i + p;
 	sd = s->data + p;
-	if (mod == 1) {
-		val += sd[0] * yd[si[0]];
-		si++;
-		sd++;
-	} else if (mod == 2) {
-		val += sd[0] * yd[si[0]] + sd[1] * yd[si[1]];
-		si += 2;
-		sd += 2;
-	} else if (mod == 3) {
-		val += sd[0] * yd[si[0]] + sd[1] * yd[si[1]] + sd[2] * yd[si[2]];
-		si += 3;
-		sd += 3;
-	}
-	for (k = mod; k < n; k += 4) {
-		val += sd[0] * yd[si[0]] + sd[1] * yd[si[1]] + sd[2] * yd[si[2]] + sd[3] * yd[si[3]];
-		si += 4;
-		sd += 4;
-	}
+	for (l = 0; l < n; l++) val += sd[l] * yk[si[l]];
 
 	if (mm_real_is_symmetric (s)) {
-		int		k0;
-		int		k1;
+		int		l0;
+		int		l1;
 		if (mm_real_is_upper (s)) {
-			k0 = j + 1;
-			k1 = s->n;
+			l0 = j + 1;
+			l1 = s->n;
 		} else {
-			k0 = 0;
-			k1 = j;
+			l0 = 0;
+			l1 = j;
 		}
 		sd = s->data;
-		for (k = k0; k < k1; k++) {
-			int		l = find_jth_row_element_of_sk (j, s, k);
+		for (l = l0; l < l1; l++) {
+			int		t = find_row_element (j, s, l);
 			// if found
-			if (l >= 0) val += sd[l] * yd[k];
+			if (t >= 0) val += sd[t] * yk[l];
 		}
 	}
 
 	return val;
 }
 
-/* d(:,j)' * y */
+/* d(:,j)' * y(:,k) */
 static double
-mm_real_dj_trans_dot_y (const mm_dense *d, const int j, const mm_dense *y)
+mm_real_dj_trans_dot_yk (const mm_dense *d, const int j, const mm_dense *y, const int k)
 {
 	double	val = 0.;
-	if (!mm_real_is_symmetric (d)) val = ddot_ (&d->m, d->data + j * d->m, &ione, y->data, &ione);
+	double	*yk = y->data + k * y->m;
+	if (!mm_real_is_symmetric (d)) val = ddot_ (&d->m, d->data + j * d->m, &ione, yk, &ione);
 	else {
 		int		n;
 		if (mm_real_is_upper (d)) {
 			n = j;
-			val = ddot_ (&n, d->data + j * d->m, &ione, y->data, &ione);
+			val = ddot_ (&n, d->data + j * d->m, &ione, yk, &ione);
 			n = d->m - j;
-			val += ddot_ (&n, d->data + j * d->m + j, &d->m, y->data + j, &ione);
+			val += ddot_ (&n, d->data + j * d->m + j, &d->m, yk + j, &ione);
 		} else if (mm_real_is_lower (d)) {
 			n = d->m - j;
-			val = ddot_ (&n, d->data + j * d->m + j, &ione, y->data + j, &ione);
+			val = ddot_ (&n, d->data + j * d->m + j, &ione, yk + j, &ione);
 			n = j;
-			val += ddot_ (&n, d->data + j, &d->m, y->data, &ione);
+			val += ddot_ (&n, d->data + j, &d->m, yk, &ione);
 		}
 	}
 	return val;
 }
 
-/*** x(:,j)' * y ***/
+/*** x(:,j)' * y(:,k) ***/
 double
+mm_real_xj_trans_dot_yk (const mm_real *x, const int j, const mm_dense *y, const int k)
+{
+	if (j < 0 || x->n <= j) error_and_exit ("mm_real_xj_trans_dot_yk", "first index out of range.", __FILE__, __LINE__);
+	if (k < 0 || y->n <= k) error_and_exit ("mm_real_xj_trans_dot_yk", "second index out of range.", __FILE__, __LINE__);
+	if (!mm_real_is_dense (y)) error_and_exit ("mm_real_xj_trans_dot_yk", "y must be dense.", __FILE__, __LINE__);
+	if (mm_real_is_symmetric (y)) error_and_exit ("mm_real_xj_trans_dot_yk", "y must be general.", __FILE__, __LINE__);
+	if (x->m != y->m) error_and_exit ("mm_real_xj_trans_dot_yk", "matrix dimensions do not match.", __FILE__, __LINE__);
+
+	return (mm_real_is_sparse (x)) ? mm_real_sj_trans_dot_yk (x, j, y, k) : mm_real_dj_trans_dot_yk (x, j, y, k);
+}
+
+/*** x(:,j)' * y ***/
+mm_dense *
 mm_real_xj_trans_dot_y (const mm_real *x, const int j, const mm_dense *y)
 {
-	if (j < 0 || x->n <= j) error_and_exit ("mm_real_xj_trans_dot_y", "index out of range.", __FILE__, __LINE__);
+	int			k;
+	mm_dense	*z;
+	if (j < 0 || x->n <= j) error_and_exit ("mm_real_xj_trans_dot_y", "first index out of range.", __FILE__, __LINE__);
 	if (!mm_real_is_dense (y)) error_and_exit ("mm_real_xj_trans_dot_y", "y must be dense.", __FILE__, __LINE__);
 	if (mm_real_is_symmetric (y)) error_and_exit ("mm_real_xj_trans_dot_y", "y must be general.", __FILE__, __LINE__);
-	if (y->n != 1) error_and_exit ("mm_real_xj_trans_dot_y", "y must be vector.", __FILE__, __LINE__);
-	if (x->m != y->m) error_and_exit ("mm_real_xj_trans_dot_y", "vector and matrix dimensions do not match.", __FILE__, __LINE__);
-
-	return (mm_real_is_sparse (x)) ? mm_real_sj_trans_dot_y (x, j, y) : mm_real_dj_trans_dot_y (x, j, y);
+	if (x->m != y->m) error_and_exit ("mm_real_xj_trans_dot_y", "matrix dimensions do not match.", __FILE__, __LINE__);
+	z = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, 1, y->n, y->n);
+#pragma omp parallel for
+	for (k = 0; k < y->n; k++) z->data[k] = mm_real_xj_trans_dot_yk (x, j, y, k);
+	return z;
 }
 
 /* y = alpha * s(:,j) + y */
@@ -1125,7 +1344,7 @@ mm_real_asjpy (const double alpha, const mm_sparse *s, const int j, mm_dense *y)
 		}
 		sd = s->data;
 		for (k = k0; k < k1; k++) {
-			int		l = find_jth_row_element_of_sk (j, s, k);
+			int		l = find_row_element (j, s, k);
 			// if found
 			if (l >= 0) yd[k] += alpha * sd[l];
 		}
@@ -1194,7 +1413,7 @@ mm_real_asjpy_atomic (const double alpha, const mm_sparse *s, const int j, mm_de
 		}
 		sd = s->data;
 		for (k = k0; k < k1; k++) {
-			int		l = find_jth_row_element_of_sk (j, s, k);
+			int		l = find_row_element (j, s, k);
 			// if found
 			if (l >= 0) atomic_add (yd + k, alpha * sd[l]);
 		}
