@@ -31,18 +31,20 @@ soft_threshold (const double z, const double gamma)
 static double
 cdescent_gradient (const cdescent *cd, const int j)
 {
-	double	cj = cd->lreg->c->data[j];	// X' * y
-	double	xjmu = mm_real_xj_trans_dot_y (cd->lreg->x, j, cd->mu);	// X(:,j)' * mu
+	double	cj = cd->lreg->c->data[j];	// c = X' * y
+	double	xjmu = mm_real_xj_trans_dot_yk (cd->lreg->x, j, cd->mu, 0);	// X(:,j)' * mu
 
 	//	z = c(j) - X(:,j)' * mu
 	double	z = cj - xjmu;
 
 	// if X is not centered and cd->b != 0, z -= sum(X(:,j)) * b
-	if (!cd->lreg->xcentered && fabs (cd->b) > 0.) z -= cd->lreg->sx[j] * cd->b;
+	if (cd->use_intercept) {
+		if (!cd->lreg->xcentered && fabs (cd->b0) > 0.) z -= cd->lreg->sx[j] * cd->b0;
+	}
 
 	// not lasso, z -= lambda2 * D(:,j)' * nu (nu = D * beta)
 	if (!cd->is_regtype_lasso)
-		z -= cd->lreg->lambda2 * mm_real_xj_trans_dot_y (cd->lreg->d, j, cd->nu);
+		z -= cd->lambda2 * mm_real_xj_trans_dot_yk (cd->lreg->d, j, cd->nu, 0);
 
 	return z;
 }
@@ -52,7 +54,7 @@ static double
 cdescent_scale2 (const cdescent *cd, const int j)
 {
 	double	scale2 = (cd->lreg->xnormalized) ? 1. : cd->lreg->xtx[j];
-	if (!cd->is_regtype_lasso) scale2 += cd->lreg->lambda2 * cd->lreg->dtd[j];
+	if (!cd->is_regtype_lasso) scale2 += cd->lambda2 * cd->lreg->dtd[j];
 	return scale2;
 }
 
